@@ -70,7 +70,7 @@ impl StrExt for &str {
     where
         CRL: AsRef<Cache> + Send + Sync,
     {
-        F::from_str(&cache, &self).await
+        F::from_str(&cache, self).await
     }
 }
 
@@ -1025,6 +1025,8 @@ mod test {
                 referenced_message: None,
                 #[cfg(feature = "unstable_discord_api")]
                 interaction: None,
+                #[cfg(feature = "unstable_discord_api")]
+                components: vec![],
             },
         };
 
@@ -1088,6 +1090,7 @@ mod test {
             let mut channels = HashMap::new();
             channels.insert(ChannelId(2), guild_channel.clone());
 
+            #[allow(deprecated)]
             GuildCreateEvent {
                 guild: Guild {
                     id: GuildId(1),
