@@ -1003,22 +1003,20 @@ impl CacheUpdate for PresenceUpdateEvent {
 
                 // Create a partial member instance out of the presence update
                 // data.
-                if !guild.members.contains_key(&self.presence.user_id) {
-                    if let Some(user) = self.presence.user.as_ref() {
-                        guild.members.insert(self.presence.user_id, Member {
-                            deaf: false,
-                            guild_id,
-                            joined_at: None,
-                            mute: false,
-                            nick: None,
-                            user: user.clone(),
-                            roles: vec![],
-                            pending: false,
-                            premium_since: None,
-                            #[cfg(feature = "unstable_discord_api")]
-                            permissions: None,
-                        });
-                    }
+                if let Some(user) = self.presence.user.as_ref() {
+                    guild.members.entry(self.presence.user_id).or_insert_with(|| Member {
+                        deaf: false,
+                        guild_id,
+                        joined_at: None,
+                        mute: false,
+                        nick: None,
+                        user: user.clone(),
+                        roles: vec![],
+                        pending: false,
+                        premium_since: None,
+                        #[cfg(feature = "unstable_discord_api")]
+                        permissions: None,
+                    });
                 }
             }
         } else if self.presence.status == OnlineStatus::Offline {
